import history_read_f
import tkinter as tk
import matplotlib
matplotlib.use('Agg')
from matplotlib import pyplot as plt
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
from matplotlib.backend_bases import key_press_handler
import numpy as np
import os
from scipy import fftpack, optimize

rho0 = 6.42e-2
aion = 2
t_unit=np.sqrt(aion)/rho0
uni_font = ("Helvetica", 20)
mpdiag = 11

#TODO: use message box to determine time range
plt.rcParams.update({'font.size': 14, 'font.family':'serif'})

class ToplevelData(tk.Toplevel):
    def __init__(self, master=None, cnf={}, **kw):
        super(ToplevelData, self).__init__(master=master, cnf=cnf, **kw)
        self.read = True
    def destroy(self):
        history_read_f.histroy_parameters.dealloc()
        super(ToplevelData, self).destroy()
        self.read = False

def read(directory):
    history_file_name = os.path.join(directory, "frc_history.out")
    print("Reading %s ..." %history_file_name)
    print('aion= %d, rho0=%f' % (aion, rho0))
    print("omega_c/omega_t= %f" % (np.sqrt(aion)/rho0))
    history_read_f.histroy_parameters.filename(history_file_name)
    history_read_f.histroy_parameters.history_read()
    tstep_diag = history_read_f.histroy_parameters.tstep_diag
    realstep = history_read_f.histroy_parameters.realstep
    print('delta_t', tstep_diag, 'steps', realstep)

def panel(window, global_win_grp,work_dir):
    # p = tk.Toplevel(window,width=300, height=200)
    p = ToplevelData(window, width=300, height=200)
    p.title("history")
    global_win_grp.append(p)
    read(work_dir)
    realstep = history_read_f.histroy_parameters.realstep
    phi = history_read_f.histroy_parameters.phi_hist
    b_phi = tk.Button(p, text='phi', command=lambda: gen_plot(window, global_win_grp, "$\delta\phi$", phi), font=uni_font)
    b_phi.grid(row=1,column=1,sticky=tk.W+tk.E)

    phir = history_read_f.histroy_parameters.phi_real_hist[:realstep]
    phii = history_read_f.histroy_parameters.phi_imag_hist[:realstep]
    b_phi_m1 = tk.Button(p, text='phi(cmplx)', command=lambda: plot_phi_mode1(window, global_win_grp, phir, phii), font=uni_font)
    b_phi_m1.grid(row=2,column=1,sticky=tk.W+tk.E)

    phimode1r = history_read_f.histroy_parameters.fieldmode[1,:realstep]
    phimode1i = history_read_f.histroy_parameters.fieldmode[0,:realstep]
    b_phi_m1 = tk.Button(p, text='phi mode1', command=lambda: plot_phi_mode1(window, global_win_grp, phimode1r, phimode1i), font=uni_font)
    b_phi_m1.grid(row=3,column=1,sticky=tk.W+tk.E)

    den_e = history_read_f.histroy_parameters.densitye
    b_den_e = tk.Button(p, text='densitye', command=lambda: gen_plot(window, global_win_grp, "$\delta n_e$", den_e), font=uni_font)
    b_den_e.grid(row=1,column=2,sticky=tk.W+tk.E)

    sdeltne = history_read_f.histroy_parameters.sdeltne
    b_sdeltne = tk.Button(p, text='sdeltne', command=lambda: gen_plot(window, global_win_grp, "sdeltne", sdeltne), font=uni_font)
    b_sdeltne.grid(row=2,column=2,sticky=tk.W+tk.E)

    flowe = history_read_f.histroy_parameters.flowe
    b_flowe = tk.Button(p, text='flowe', command=lambda: gen_plot(window, global_win_grp, "$\detla u_e$", flowe), font=uni_font)
    b_flowe.grid(row=3,column=2,sticky=tk.W+tk.E)

    suepara = history_read_f.histroy_parameters.suepara
    b_suepara = tk.Button(p, text='suepara', command=lambda: gen_plot(window, global_win_grp, "suepara", suepara), font=uni_font)
    b_suepara.grid(row=4,column=2,sticky=tk.W+tk.E)

    b_part = tk.Button(p, text='particle', command=lambda: plot_part(window, global_win_grp,work_dir), font=uni_font)
    b_part.grid(row=1,column=3,sticky=tk.W+tk.E)
 
    phi_rms =  history_read_f.histroy_parameters.phi_rms
    b_phi_rms = tk.Button(p, text='phi_rms', command=lambda: gen_plot(window, global_win_grp, "$\phi_{RMS}$", phi_rms), font=uni_font)
    b_phi_rms.grid(row=2,column=3,sticky=tk.W+tk.E)
    
    phi0_rms =  history_read_f.histroy_parameters.phi0_rms
    b_phi0_rms = tk.Button(p, text='phi0_rms', command=lambda: gen_plot(window, global_win_grp, r"$\langle\phi\rangle_{RMS}$", phi0_rms), font=uni_font)
    b_phi0_rms.grid(row=3,column=3,sticky=tk.W+tk.E)

    b_close = tk.Button(p, text='close', command=p.destroy, font=uni_font)
    b_close.grid(row=1,column=4,sticky=tk.W+tk.E)

    p.protocol("WM_DELETE_WINDOW", p.destroy)
    return p

def fft(tsize, steps, phi_real, phi_imag):
    tstep_diag = tsize
    x = phi_real + 1j * phi_imag
    y = fftpack.fft(x)
    y = np.fft.fftshift(y)
    frange = np.linspace(-np.pi/tstep_diag, np.pi*(steps-2)/tstep_diag/steps ,steps)
    return frange, y
    
def plot_part(window, global_win_grp, directory='.'):
    
    part_file = os.path.join(directory, "partdiag.out")
    part_info = np.loadtxt(part_file)
    tstep_diag = part_info[1]
    nspecies = int(part_info[2])

    ndstep = (len(part_info) - 3) // (mpdiag * nspecies)
    trange = np.arange(ndstep) * tstep_diag / t_unit
    trim_info = part_info[3:(ndstep*mpdiag*nspecies)+3]
    part_info_pack = (trim_info).reshape((-1, nspecies, mpdiag))
    df_ion = part_info_pack[:,0,0]
    df2_ion = part_info_pack[:,0,1]
    ff_energy_ion = part_info_pack[:,0,4]
    df_energy_ion = part_info_pack[:,0,5]
    ff_angmom_ion = part_info_pack[:,0,2]
    df_angmom_ion = part_info_pack[:,0,3]
    pflux_ion = part_info_pack[:,0,6]
    mflux_ion = part_info_pack[:,0,7]
    eflux_ion = part_info_pack[:,0,8]

    df_e = part_info_pack[:,1,0]
    df2_e = part_info_pack[:,1,1]
    ff_energy_e = part_info_pack[:,1,4]
    df_energy_e = part_info_pack[:,1,5]
    ff_angmom_e = part_info_pack[:,1,2]
    df_angmom_e = part_info_pack[:,1,3]
    pflux_e = part_info_pack[:,1,6]
    mflux_e = part_info_pack[:,1,7]
    eflux_e = part_info_pack[:,1,8]

    new_win = tk.Toplevel(master=window)
    new_win.wm_title("ion particle info")
    fig = plt.figure()
    fig.tight_layout()
    ax = fig.add_subplot(421)
    ax.plot(trange,df_ion)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r'ion $\delta f$')
    ax = fig.add_subplot(422)
    ax.plot(trange, df2_ion)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$\delta^2 f$")
    ax = fig.add_subplot(423)
    ax.plot(trange, ff_energy_ion)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$E_k$")
    ax = fig.add_subplot(424)
    ax.plot(trange, df_energy_ion)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$\delta f E_k$")
    ax = fig.add_subplot(425)
    ax.plot(trange, ff_angmom_ion)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$P_\zeta$")
    ax = fig.add_subplot(426)
    ax.plot(trange, df_angmom_ion)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$\delta f P_\zeta$")
    ax = fig.add_subplot(427)
    ax.plot(trange, pflux_ion)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$Pflux_i$")
    ax = fig.add_subplot(428)
    ax.plot(trange, eflux_ion)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$Eflux_i$")
    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top",fill='both',expand=True)

    new_wine = tk.Toplevel(master=window)
    new_wine.wm_title("electron particle info")
    fig2 = plt.figure()
    fig2.tight_layout()
    ax = fig2.add_subplot(421)
    ax.plot(trange,df_e)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r'electron $\delta f$')
    ax = fig2.add_subplot(422)
    ax.plot(trange, df2_e)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$\delta^2 f$")
    ax = fig2.add_subplot(423)
    ax.plot(trange, ff_energy_e)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$E_k$")
    ax = fig2.add_subplot(424)
    ax.plot(trange, df_energy_e)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$\delta f E_k$")
    ax = fig2.add_subplot(425)
    ax.plot(trange, ff_angmom_e)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$P_\zeta$")
    ax = fig2.add_subplot(426)
    ax.plot(trange, df_angmom_e)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$\delta f P_\zeta$")
    ax = fig2.add_subplot(427)
    ax.plot(trange, pflux_e)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$Pflux_e$")
    ax = fig2.add_subplot(428)
    ax.plot(trange, eflux_e)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1))
    ax.set_title(r"$Eflux_e$")
    fig2.tight_layout()
    canvas2 = FigureCanvasTkAgg(fig2, master=new_wine)
    canvas2.draw()

    toolbar2 = NavigationToolbar2Tk(canvas2, new_wine)
    toolbar2.update()
    canvas2.get_tk_widget().pack(side="top",fill='both',expand=True)


def f_1(x,A,B):
    return A*x+B
def f_2(x,A,B,C):
    return A*np.cos(B*x+C)

def plot_phi_mode1(window, global_win_grp, rvalue, ivalue):
    from picker import Picker
    realstep = history_read_f.histroy_parameters.realstep
    tstep_diag = history_read_f.histroy_parameters.tstep_diag
    time_range = list(range(realstep)) * tstep_diag / t_unit

    new_win = tk.Toplevel(window)
    new_win.wm_title("phi history mode1")
    fig = plt.figure()

    ax = fig.add_subplot(221)
    ax.plot(time_range, rvalue)
    ax.plot(time_range, ivalue)
    ax.legend([r"$\Re{\phi}$", r"$\Im{\phi}$"])

    ax3 = fig.add_subplot(222)
    ax4 = fig.add_subplot(223)

    length=len(time_range)
    start=tk.IntVar()
    start.set(int(length//10*4))
    end=tk.IntVar()
    end.set(int(length//10*8))
    omega0 = tk.DoubleVar()
    omega0.set(0.1)

    phi_mod = np.sqrt(rvalue**2 + ivalue**2)
    phi_log = np.log(phi_mod)
    ax3.plot(phi_log, 'k-')
    A1, B1 = plotfitlog(ax3, time_range, phi_log, start, end)
    

    norm_real = rvalue / (np.exp(A1*time_range+B1))
    norm_imag = ivalue / (np.exp(A1*time_range+B1))
    ax4.plot(time_range, norm_real,'k-')
    ax4.plot(time_range, norm_imag,'b-')
    A2, B2, C2 = plotfitsin(ax4,time_range, norm_real, start, end, omega0)

    ax2 = fig.add_subplot(224)
    tsize = history_read_f.histroy_parameters.tstep_diag / t_unit
    freq, y = fft(tsize, realstep, rvalue, ivalue)
    # -i\omega*t rather than i\omega*t
    freq = -freq
    maxloc = np.argmax(np.absolute(y))
    maxfreq = freq[maxloc]
    ax2.plot(freq, np.absolute(y))
    ax2.set_xlim([-5,5])
    ax2.set_title(r"$\omega=%f$" % maxfreq)
    

    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top",fill='both',expand=True)

    b_time = tk.Button(new_win, text='change_time', command=lambda: change_time_pop(new_win, canvas, ax3, ax4, time_range, rvalue, ivalue, start, end, omega0), font=uni_font)
    b_time.pack(side="left",expand=False)
    norm_data = np.stack((norm_real, norm_imag), axis=0)
    b_omega = tk.Button(new_win, text='omega', command=lambda: change_omega_pop(new_win, canvas, ax4, time_range, norm_data, start, end, omega0), font=uni_font)
    b_omega.pack(side='left',expand=False)

def plotfitlog(ax2, time_range, phi_log, start, end):
    A1, B1 = optimize.curve_fit(f_1, time_range[start.get():end.get()], phi_log[start.get():end.get()])[0]
    ax2.plot(np.array([start.get(),end.get()]), A1*time_range[[start.get(),end.get()]]+B1,'ro-')
    ax2.set_title(r"$\gamma=%f$" % A1)
    return A1, B1

def plotfitsin(ax3, time_range, norm_real, start, end, omega0):
    A2,B2,C2 = optimize.curve_fit(f_2, time_range[start.get():end.get()], norm_real[start.get():end.get()],p0=[1,omega0.get(),1])[0]
    ax3.plot(time_range[start.get():end.get()], A2*np.cos(B2*time_range[start.get():end.get()]+C2),'r--')
    ax3.plot(time_range[[start.get(),end.get()]], A2*np.cos(B2*time_range[[start.get(),end.get()]]+C2),'ro')
    ax3.set_title(r"$\omega=%f$" % B2)
    return A2, B2, C2
 

def change_omega_pop(root, canvas, ax, time_range, data, start, end, omega0):
    pop = tk.Toplevel(root)
    # info = "Reset max and min value for color"
    l1 = tk.Label(pop, text="init omega")
    omega_entry = tk.Entry(pop, textvariable=omega0)
    l1.grid(row=1, column=1)
    omega_entry.grid(row=1, column=2, columnspan=2)
    b_set = tk.Button(pop, text='Set', command=lambda: change_omega(canvas, pop, ax, omega0, time_range, data, start, end))
    b_set.grid(row=2, column=1)

def change_omega(canvas, pop, ax, omega0, time_range, data, start, end):
    ax.clear()
    ax.plot(time_range, data[0,:], 'k-')
    ax.plot(time_range, data[1,:], 'b-')
    A2, B2, C2 = plotfitsin(ax, time_range, data[0,:], start, end, omega0)
#    length = len(time_range)
#    A2,B2,C2 = optimize.curve_fit(f_2, time_range[start.get():end.get()], data[0,start.get():end.get()],p0=[1,omega,1])[0]
#    ax.plot(time_range, A2*np.cos(B2*time_range+C2),'r--')
    ax.set_title(r"$\omega=%f$" % B2)
    canvas.draw()
    pop.destroy()
def change_time_pop(root, canvas, ax2, ax3, time_range, phi_real, phi_imag, start, end, omega0):
    pop = tk.Toplevel(root)
    lstart = tk.Label(pop, text="start")
    lend = tk.Label(pop, text="end")
    start_entry = tk.Entry(pop, textvariable=start)
    end_entry = tk.Entry(pop, textvariable=end)
    lstart.grid(row=1, column=1)
    start_entry.grid(row=1, column=2, columnspan=2)
    lend.grid(row=2,column=1)
    end_entry.grid(row=2, column=2, columnspan=2)
    b_set = tk.Button(pop, text='Set', command=lambda: change_time(canvas, pop, ax2, ax3, time_range, phi_real, phi_imag, start, end, omega0))
    b_set.grid(row=3, column=1)
    

def change_time(canvas, pop, ax2, ax3, time_range, phi_real, phi_imag, start, end, omega0):
    phi_mod = np.abs(phi_real+1j*phi_imag)
    phi_log = np.log(phi_mod)
    ax2.clear()
    ax2.plot(phi_log, 'k-')
    A1, B1 = plotfitlog(ax2, time_range, phi_log, start, end)
    norm_real = phi_real / (np.exp(A1*time_range+B1))
    norm_imag = phi_imag / (np.exp(A1*time_range+B1))
    ax3.clear()
    ax3.plot(time_range, norm_real,'k-')
    ax3.plot(time_range, norm_imag,'b-')
    A2, B2, C2 = plotfitsin(ax3, time_range, norm_real, start, end, omega0)
    canvas.draw() 
    pop.destroy()
    
    
    

def gen_plot(window, global_win_grp, title, values):
    realstep = history_read_f.histroy_parameters.realstep
    length=realstep
    start=tk.IntVar()
    start.set(int(length//10*4))
    end=tk.IntVar()
    end.set(int(length//10*8))

    tstep_diag = history_read_f.histroy_parameters.tstep_diag
    time_range = list(range(realstep)) * tstep_diag / t_unit

    new_win = tk.Toplevel(window)

    fig = plt.figure()
    ax = fig.add_subplot(211)
    value_plot = values[:realstep]
    ax.plot(time_range, value_plot)
    ax.set_title(title)
    ax.set_xlabel("$t [R_0/C_s]$", fontsize=14)
    ax.yaxis.get_major_formatter().set_powerlimits((-1, 1)) 

    ax2 = fig.add_subplot(212)
    log_plot = np.log(np.abs(values[:realstep]))

    ax2.plot(log_plot)
    plotfitlog(ax2, time_range, log_plot, start, end)
    


    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top",fill='both',expand=True)

    b_time = tk.Button(new_win, text='change_time', command=lambda: change_time_pop2(new_win, canvas, ax2, time_range, log_plot, start, end), font=uni_font)
    b_time.pack(side="left",expand=False)

def change_time_pop2(root, canvas, ax2, time_range, log_plot, start, end):
    pop = tk.Toplevel(root)
    lstart = tk.Label(pop, text="start")
    lend = tk.Label(pop, text="end")
    start_entry = tk.Entry(pop, textvariable=start)
    end_entry = tk.Entry(pop, textvariable=end)
    lstart.grid(row=1, column=1)
    start_entry.grid(row=1, column=2, columnspan=2)
    lend.grid(row=2,column=1)
    end_entry.grid(row=2, column=2, columnspan=2)
    b_set = tk.Button(pop, text='Set', command=lambda: change_time2(canvas, pop, ax2, time_range, log_plot, start, end))
    b_set.grid(row=3, column=1)


def change_time2(canvas, pop, ax2, time_range, log_plot, start, end):
    ax2.clear()
    ax2.plot(log_plot, 'k-')
    A1, B1 = plotfitlog(ax2, time_range, log_plot, start, end)
    canvas.draw()
    pop.destroy()
